#pragma once
#include <iostream>
#include <vector>
#include <array>
#include <set>
#include <unordered_set>
#include <algorithm>

struct Node {
	int ID_Node = 1;
	std::array<double, 3> Coordinate;
	bool flag = true;
	Node() = default;
	Node(int ID_Node, const std::array<double, 3>& Coordinate, bool flag):
		ID_Node(ID_Node), Coordinate(Coordinate), flag(flag) {}

	bool operator < (const Node& elem) const {
		return (ID_Node < elem.ID_Node);
	}
	friend std::ostream& operator << (std::ostream& out, const std::vector <Node>& elem);
};


struct FiniteElement {
	int ID_FiniteElement = 1;
	int ID_Material = 1;
	std::vector<int> ID_Nodes;
	bool operator < (const FiniteElement& elem) const {
		return (ID_FiniteElement < elem.ID_FiniteElement);
	}
	friend std::ostream& operator << (std::ostream& out, const std::vector <FiniteElement>& elem);
	friend std::ostream& operator << (std::ostream& out, const std::set <FiniteElement>& elem);
};

struct BoundaryFiniteElement {
	int ID_Element = 1;
	int ID_Boundary = 1;
	std::vector<int> ID_Nodes;
	friend std::ostream& operator << (std::ostream& out, const std::vector <BoundaryFiniteElement>& elem);

};

struct Edge{
	int first_id;
	int last_id;
	int middle_id = 0;
	Edge() = default;
	Edge(int id1, int id2) : first_id(id1), last_id(id2) {}
	bool operator == (const Edge& elem) const {
		if (first_id == elem.first_id && last_id == elem.last_id) return true;
		else return (first_id == elem.last_id && last_id == elem.first_id);
	}
};


class EdgeHash{
public:
	std::size_t operator() (const Edge& _p) const
	{
		std::size_t o_seed = 0;
		if (_p.first_id > _p.last_id)
		{
			o_seed ^= std::hash<int>() (_p.first_id) + 0x9e3779b9 + (o_seed << 6) + (o_seed >> 2);
			o_seed ^= std::hash<int>() (_p.last_id) + 0x9e3779b9 + (o_seed << 6) + (o_seed >> 2);
		}
		else
		{
			o_seed ^= std::hash<int>() (_p.last_id) + 0x9e3779b9 + (o_seed << 6) + (o_seed >> 2);
			o_seed ^= std::hash<int>() (_p.first_id) + 0x9e3779b9 + (o_seed << 6) + (o_seed >> 2);
		}

		return o_seed;
	};
};
